package de.fitko.fitconnect.client;

import com.google.inject.Guice;
import com.google.inject.Injector;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.Submission;
import de.fitko.fitconnect.api.services.Sender;
import de.fitko.fitconnect.api.services.metadata.MetadataService;
import de.fitko.fitconnect.dependency.SdkModule;

import java.util.Optional;

/**
 * A fluent client for handing in a {@link Submission}
 */
public class SenderClient {

    private static final Injector injector = Guice.createInjector(new SdkModule());

    private SenderClient() {}

     public static Authenticate builder() {
         Sender sender = injector.getInstance(Sender.class);
         MetadataService metadataService = injector.getInstance(MetadataService.class);
         return new ClientBuilder(sender, metadataService);
    }

    public interface Authenticate {
        /**
         *
         * @param clientId
         * @param secret
         * @param scope
         * @return
         */
        CreateSubmission authenticate(String clientId, String secret, String... scope);
    }

    public interface AttachmentUpload {
        SendSubmission uploadAttachments();
    }

    public interface CreateSubmission {
        AttachmentUpload createSubmission(Submission submission);
    }

    public interface SendSubmission {
        Submission sendSubmission(Metadata metaData, Data data);
    }

    public static class ClientBuilder implements Authenticate, AttachmentUpload, CreateSubmission, SendSubmission {

        private final MetadataService metadataService;
        private Sender sender;
        private Submission submission;
        private Optional<OAuthToken> token;

        private ClientBuilder(Sender sender, MetadataService metadataService) {
            this.sender = sender;
            this.metadataService = metadataService;
        }

        @Override
        public CreateSubmission authenticate(String clientId, String secret, String... scope) {
            this.token = sender.retrieveOAuthToken(clientId, secret, scope);
            return this;
        }

        @Override
        public SendSubmission uploadAttachments() {
            checkIfAuthenticated();
            //metadataService.createMetadata();
            return this;
        }

        @Override
        public AttachmentUpload createSubmission(Submission submission) {
            checkIfAuthenticated();
            sender.sendSubmission(submission);
            return this;
        }

        @Override
        public Submission sendSubmission(Metadata metaData, Data data) {
            checkIfAuthenticated();
            return sender.sendSubmission(Submission.builder().build());
        }

        private void checkIfAuthenticated(){
            if(this.token == null || this.token.isEmpty()){
                throw new IllegalStateException("Not authenticated, please authenticate first");
            }
        }
    }
}